// Martin Thomas 3/2008

#include <stdint.h>
#include <stdlib.h>
#include <avr/io.h>

#include "../../BaseTinyFirmware/GCC/enums.h"
#include "../../BaseTinyFirmware/GCC/structs.h"

#include "spi.h"
#include "bc100_slave.h"

#ifdef BC100_SLAVE_TEST
#include "mystdio.h"
#endif

#define dummy 0x00

void bc100_slave_init( void )
{
}

uint8_t bc100_slave_read( uint8_t cmd, size_t size, void *buf )
{
	uint8_t err = 0;

	return err;
}

#ifdef BC100_SLAVE_TEST

uint8_t bc100_slave_test(void)
{
	uint8_t err = 0;
	uint8_t in, out, i;
	uint32_t timevals[TIMERS];

	SPI_init();

	SPI_rw( dummy );
	myprintf_P("Timers:\n");
	out = BC100_SLAVE_READ | BC100_SLAVE_SRAM | sizeof(timevals);
	SPI_rw( out );
	out = ADR_TIMERS;
	in = SPI_rw( out );
	myprintf_P("In1 : 0x%02x (expected 0xcc)\n", in );
	in = SPI_rw( dummy );
	myprintf_P("In2 : 0x%02x (expected 0xbb)\n", in );
#if 0
	uint8_t tmp
	for ( i = TIMERS; i > 0; i-- ) {
		tmp  = (uint32_t)(SPI_rw(dummy)) << 24;
		tmp |= (uint32_t)(SPI_rw(dummy)) << 16;
		tmp |= (uint32_t)(SPI_rw(dummy)) <<  8;
		tmp |= (uint32_t)(SPI_rw(dummy)) <<  0;
		timevals[i-1] = tmp;
	}
#else
	i = sizeof(timevals);
	while ( i-- > 0 ) {
		*( (unsigned char*)&timevals + i ) = SPI_rw(dummy);
	}
#endif
	for ( i=0; i<TIMERS; i++ ) {
		myprintf_P("Timer #%d = %lu (0x%08lx)\n", i, timevals[i], timevals[i] );
	}


	ADC_Status_t ADCS;
	myprintf_P("ADC-Status:\n");
	out = BC100_SLAVE_READ | BC100_SLAVE_SRAM | sizeof(ADCS);
	SPI_rw( out );
	out = ADR_ADCS;
	in = SPI_rw( out );
	myprintf_P("In1 : 0x%02x (expected 0xcc)\n", in );
	in = SPI_rw( dummy );
	myprintf_P("In2 : 0x%02x (expected 0xbb)\n", in );
	i = sizeof(ADCS);
	while ( i-- > 0 ) {
		*( (unsigned char*)&ADCS + i ) = SPI_rw(dummy);
	}
	myprintf_P("ADC-MUX        %d\n", ADCS.MUX );
	myprintf_P("Cycle complete %d\n", ADCS.Flag);
	myprintf_P("Mains OK       %d\n", ADCS.Mains);
	myprintf_P("A/D halt       %d\n", ADCS.Halt);
	myprintf_P("Offs ADC3 g20  %d\n", ADCS.ADC3_G20_OS);
	myprintf_P("Offs ADC5 g20  %d\n", ADCS.ADC5_G20_OS);
	myprintf_P("rawRID         %d\n", ADCS.rawRID);
	myprintf_P("rawNTC         %d\n", ADCS.rawNTC);
	myprintf_P("rawVBAT        %d\n", ADCS.rawVBAT);
	myprintf_P("Supply         %d [mV]\n", ADCS.VIN);
	myprintf_P("Batt. Voltage  %d [mV]\n", ADCS.VBAT);
	myprintf_P("Batt. Current  %d [mA]\n", ADCS.IBAT);
	for ( i=0; i<4; i++ ) {
		myprintf_P("DiscIBAT[%d]    %d [mA]\n", i, ADCS.discIBAT[i] );
	}
	myprintf_P("average IBAT   %d [mA]\n", ADCS.avgIBAT);

	Batteries_t BattData;
	myprintf_P("Current Battery:\n");
	out = BC100_SLAVE_READ | BC100_SLAVE_SRAM | sizeof(BattData);
	SPI_rw( out );
	out = ADR_BATTDATA;
	in = SPI_rw( out );
	myprintf_P("In1 : 0x%02x (expected 0xcc)\n", in );
	in = SPI_rw( dummy );
	myprintf_P("In2 : 0x%02x (expected 0xbb)\n", in );
	i = sizeof(BattData);
	while ( i-- > 0 ) {
		*( (unsigned char*)&BattData + i ) = SPI_rw(dummy);
	}
	if ( BattData.Present ) {
		myprintf_P("Battery found\n");
		if ( BattData.Charged ) {
			myprintf_P("- fully charged\n");
		}
		if ( BattData.Low ) {
			myprintf_P("- low voltage\n");
		}
		if ( BattData.Exhausted) {
			myprintf_P("- exhausted\n");
		}
		if ( BattData.HasRID) {
			myprintf_P("- has a ID-Resistor\n");
		}
		myprintf_P("Safety circuit ID    %d\n", BattData.Circuit );
		myprintf_P("Tempearture          %d centi-degree C\n", BattData.Temperature );
		myprintf_P("ADCSteps/half degree %d\n", BattData.ADCSteps);
		myprintf_P("Capacity             %d mAh\n", BattData.Capacity);
		myprintf_P("max. Current         %d mA\n", BattData.MaxCurrent);
		myprintf_P("max. cut-off Time    %d min\n", BattData.MaxTime);
		myprintf_P("min. cut-off Current %d mA\n", BattData.MinCurrent);
	}
	else {
		myprintf_P("Battery not found\n");
	}

	SPI_release();

	return err;
}

#endif /* BC100_SLAVE_TEST */

