// M. Thomas 11/2007

#include <avr/io.h>
#include <avr/interrupt.h>
#include "key_io.h"

/* define externs */
// debounced and inverted key state, bit = 1: key pressed
volatile uint8_t key_io_state;		
// key press detect
volatile uint8_t key_io_press;
// key long press and repeat
volatile uint8_t key_io_rpt;

void key_io_init(void)
{
	KEY_IO_KEYSA_DDR &= ~( 
		(1<<KEY_IO_KEYA1_BIT) | (1<<KEY_IO_KEYA2_BIT) |
		(1<<KEY_IO_KEYA3_BIT) | (1<<KEY_IO_KEYA4_BIT) );
	KEY_IO_KEYSB_DDR &= ~(1<<KEY_IO_KEYB1_BIT);
	// external pull-ups avail. for all keys -> no init here
	key_io_state = key_io_state_hardware();
}

uint8_t key_io_get_press( uint8_t key_mask )
{
	uint8_t sreg;

	sreg = SREG;
	cli();

	key_mask &= key_io_press; // read key(s)
	key_io_press ^= key_mask; // clear key(s)

	SREG = sreg;

	return key_mask;
}

uint8_t key_io_get_rpt( uint8_t key_mask )
{
	uint8_t sreg;

	sreg = SREG;
	cli();

	key_mask &= key_io_rpt;  // read key(s)
	key_io_rpt ^= key_mask;  // clear key(s)
	
	SREG = sreg;

	return key_mask;
}

uint8_t key_io_get_was_short( uint8_t key_mask )
{
	uint8_t sreg, ret;

	sreg=SREG;
	// read key state and key press atomic !
	cli();

	ret =  key_io_get_press( ~key_io_state & key_mask );

	SREG = sreg;

	return ret;

}

uint8_t key_io_get_longpress( uint8_t key_mask )
{
	return key_io_get_press( key_io_get_rpt( key_mask ));
}


