// Copyright (c) 2007, 2008 Martin Thomas - BSD-license
// 10 ms timebase with 
// 8bit timer-counter #0 in CTC-mode on ATmega644

#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "timebase.h"

#include "key_io.h" /* keys callback/debouce */

#define COUNTER_IS_16BIT   0
#define PRESCALER   1024UL

#if TIMEBASE_DELTAT_MS != 10
#warning "code prepared for 10ms tick"
#endif

#define TIMERFREQ   (1000/TIMEBASE_DELTAT_MS) /* Hz */
#define OCRMATCHVAL ( (F_CPU*10/PRESCALER/TIMERFREQ+5)/10 - 1 )

#if COUNTER_IS_16BIT
 /* 16 bit counter */
#define OCR_MATCH_MAX 0xfffe
#else
/* 8 bit counter */
#define OCR_MATCH_MAX 0xfe
#endif /* COUNTER_IS_16BIT */
#define OCR_MATCH_MIN 0x02

#if ( OCRMATCHVAL > OCR_MATCH_MAX )
#error "PRESCALER too small or F_CPU too high"
#endif
#if ( OCRMATCHVAL < 2 )
#error "PRESCALER too large or F_CPU too low"
#endif

volatile uint16_t timebase_tick;

ISR(TIMER0_COMPA_vect)
{
	timebase_tick++;

	key_io_callback();
}

void timebase_init(void)
{
	uint8_t sreg;

	sreg=SREG;
	cli();

	// init Timer 0 Mode 4 - CTC with interrupt on compare-match 
	
	TCCR0A = (1<<WGM01);
#if (PRESCALER == 1024)
	TCCR0B = (1<<CS02) | (1<<CS00);
#else
#error "PRESCALER settings not available"
#endif
	
	OCR0A  = OCRMATCHVAL; // match-value
	
	TCNT0  = 0;           // reset timer-counter
	TIFR0  = (1<<OCF0A);  // clear interrupt
	TIMSK0 = (1<<OCIE0A); // enabled matchA interrupt

	timebase_tick = 0;

	SREG=sreg;
}

uint16_t timebase_get_tick(void)
{
	uint8_t sreg;
	uint16_t res;

	sreg = SREG;
	cli();

	res = timebase_tick;

	SREG=sreg;

	return res;
}
