/* This file has been prepared for Doxygen automatic documentation generation.*/
/*! \file *********************************************************************
 *
 * \brief
 *      State menu definition
 * 
 *      Contains the definition of the state menu.\n
 *      The state menu contains all states and adresses to associated functions.
 *
 * \par Application note:
 *      AVR458: Charging Li-Ion Batteries with BC100 \n
 *      AVR463: Charging NiMH Batteries with BC100
 *
 * \par Documentation:
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 *
 * $Name$
 * $Revision: 2299 $
 * $RCSfile$
 * $URL: http://svn.norway.atmel.com/AppsAVR8/avr458_Charging_Li-Ion_Batteries_with_BC100/tag/20070904_release_1.0/code/IAR/menu.c $
 * $Date: 2007-08-23 12:55:51 +0200 (to, 23 aug 2007) $\n
 ******************************************************************************/

#include <stdlib.h>

#include "statefunc.h"
#include "charge.h"
#include "main.h"
#include "menu.h"


//******************************************************************************
// State menu (relies on the proper battery type to be defined in main.h!)
//******************************************************************************
#ifdef NIMH
/*! \brief The state menu
 *
 * Contains all the defined states and addresses to their associated functions.
 */
__flash const MENU_STATE_t menu_state[] = {
//  State					State function
  { ST_INIT,            Initialize},
  { ST_BATCON,          BatteryControl},
  { ST_PREQUAL,         Charge},
  { ST_SLEEP,           Sleep},
  { ST_FASTCHARGE,      Charge},
  { ST_LOWRATECHARGE,   Charge},
  { ST_ENDCHARGE,       Charge},	
  { ST_DISCHARGE,       Discharge},
  { ST_ERROR,           Error},
  { 0,                  NULL},
};
#endif // NIMH

#ifdef LIION
/*! \brief The state menu
 *
 * Contains all the defined states and addresses to their associated functions.
 */
__flash const MENU_STATE_t menu_state[] = {
//  State					State function
  { ST_INIT,            Initialize},
  { ST_BATCON,          BatteryControl},
  { ST_PREQUAL,         Charge},
  { ST_SLEEP,           Sleep},
  { ST_CCURRENT,        Charge},
  { ST_CVOLTAGE,        Charge},
  { ST_ENDCHARGE,       Charge},	
  { ST_DISCHARGE,       Discharge},
  { ST_ERROR,           Error},
  { 0,                  NULL},
};
#endif // LIION
